package eu.jonahbauer.json;

import eu.jonahbauer.json.tokenizer.token.JsonToken;
import lombok.Getter;
import lombok.RequiredArgsConstructor;
import lombok.experimental.Accessors;
import org.jetbrains.annotations.Contract;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

/**
 * Java representation of a JSON boolean.
 */
@Getter
@Accessors(fluent = true)
@RequiredArgsConstructor
public enum JsonBoolean implements JsonValue, JsonToken {
    TRUE(true),
    FALSE(false),
    ;

    /**
     * Converts the given boolean to a JSON boolean.
     * @param bool a boolean
     * @return a JSON boolean
     */
    public static @NotNull JsonBoolean valueOf(boolean bool) {
        return bool ? TRUE : FALSE;
    }

    /**
     * Converts the given boolean to a JSON boolean.
     * @param bool a boolean
     * @return a JSON boolean
     */
    @Contract("null -> null; !null -> !null")
    public static @Nullable JsonBoolean valueOf(@Nullable Boolean bool) {
        return bool == null ? null : valueOf((boolean) bool);
    }

    private final boolean value;

    @Override
    public @NotNull String toString() {
        return String.valueOf(value);
    }
}
