package eu.jonahbauer.json;

import eu.jonahbauer.json.exceptions.JsonConversionException;
import lombok.AccessLevel;
import lombok.RequiredArgsConstructor;
import org.jetbrains.annotations.NotNull;

@SuppressWarnings("preview")
@RequiredArgsConstructor(access = AccessLevel.PRIVATE)
public final class JsonTemplateProcessor<T extends JsonValue> implements StringTemplate.Processor<T, RuntimeException> {
    /**
     * Parses the string template as a JSON using {@link JsonValue#parse(StringTemplate)}.
     */
    public static final @NotNull JsonTemplateProcessor<JsonValue> JSON = new JsonTemplateProcessor<>(JsonValue.class, true);
    /**
     * Parses the string template as a JSON using {@link JsonValue#parse(StringTemplate)}. Throws a
     * {@link JsonConversionException} if the result is not a JSON object.
     */
    public static final @NotNull JsonTemplateProcessor<@NotNull JsonObject> JSON_OBJECT = new JsonTemplateProcessor<>(JsonObject.class, false);
    /**
     * Parses the string template as a JSON using {@link JsonValue#parse(StringTemplate)}. Throws a
     * {@link JsonConversionException} if the result is not a JSON array.
     */
    public static final @NotNull JsonTemplateProcessor<@NotNull JsonArray> JSON_ARRAY = new JsonTemplateProcessor<>(JsonArray.class, false);

    private final @NotNull Class<T> clazz;
    private final boolean allowNull;

    @Override
    public T process(@NotNull StringTemplate template) throws RuntimeException {
        var result = JsonValue.parse(template);
        if (result == null && !allowNull || result != null && !clazz.isInstance(result)) {
            throw new JsonConversionException("The template does not represent a " + clazz.getSimpleName());
        }
        return clazz.cast(result);
    }
}
