package eu.jonahbauer.json.exceptions;

import lombok.Getter;
import org.jetbrains.annotations.NotNull;

@Getter
abstract class JsonReaderException extends JsonProcessingException {
    private final int fragment;
    private final int lineNumber;
    private final int columnNumber;

    public JsonReaderException(int fragment, int line, int column, @NotNull String message) {
        super(message);
        this.fragment = fragment;
        this.lineNumber = line;
        this.columnNumber = column;
    }

    public JsonReaderException(int fragment, int line, int column, @NotNull String message, @NotNull Throwable cause) {
        super(message, cause);
        this.fragment = fragment;
        this.lineNumber = line;
        this.columnNumber = column;
    }

    @Override
    public @NotNull String getMessage() {
        if (getFragment() == 0) {
            return super.getMessage() + " at line " + getLineNumber() + ", column " + getColumnNumber();
        } else {
            return super.getMessage() + " at fragment " + getFragment() + ", line " + getLineNumber() + ", column " + getColumnNumber();
        }
    }
}
