package eu.jonahbauer.json.tokenizer.reader;

import org.jetbrains.annotations.Nullable;
import org.jetbrains.annotations.Range;

import java.io.IOException;

public interface PushbackReader extends AutoCloseable {
    int EOF = -1;
    int PLACEHOLDER = -2;

    /**
     * Reads the next character.
     * @return the next character or {@link #EOF} if the end of the stream has been reached
     * @throws IOException if an I/O error occurs
     */
    @Range(from = -2, to = Character.MAX_VALUE) int read() throws IOException;

    /**
     * Pushes the reader back, making {@link #read()} return the same character as before.
     * @throws IllegalStateException when called before the first read or more than once after each read
     */
    void pushback();

    /**
     * {@return the index of the fragment the current character belongs to} Returns 0 unless a {@code StringTemplate} is
     * being read.
     * @throws IllegalStateException when called before the first read
     */
    @Range(from = 0, to = Integer.MAX_VALUE) int getFragment();

    /**
     * {@return the line number of the current character}
     * @throws IllegalStateException when called before the first read
     */
    @Range(from = 1, to = Integer.MAX_VALUE) int getLineNumber();

    /**
     * {@return the column number of the current character}
     * @throws IllegalStateException when called before the first read
     */
    @Range(from = 1, to = Integer.MAX_VALUE) int getColumnNumber();

    /**
     * Return the object that has been read after {@link #read()} returned {@link #PLACEHOLDER}.
     * @throws IllegalStateException the previous call to read did not return {@link #PLACEHOLDER}
     * @return the object that has been read
     */
    @Nullable Object getObject();

    @Override
    void close() throws IOException;
}
