package eu.jonahbauer.json.parser;

import eu.jonahbauer.json.exceptions.JsonParserException;
import org.jetbrains.annotations.NotNull;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.Arguments;
import org.junit.jupiter.params.provider.MethodSource;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.Objects;
import java.util.stream.Stream;

import static org.junit.jupiter.api.Assertions.*;

class JsonParserTest {

    // https://github.com/nst/JSONTestSuite
    @ParameterizedTest(name = "{0}")
    @MethodSource("parameters")
    void suite(@NotNull String name) throws IOException {
        var path = "/nst/JsonTestSuite/" + name;
        Boolean expected = switch (name.charAt(0)) {
            case 'i' -> null;
            case 'y' -> true;
            case 'n' -> false;
            default -> throw new IllegalArgumentException();
        };

        try (
                var in = Objects.requireNonNull(JsonParserTest.class.getResourceAsStream(path));
                var reader = new InputStreamReader(in)
        ) {
            var parser = new JsonParser(reader);

            if (expected == Boolean.FALSE) {
                assertThrows(JsonParserException.class, parser::parse).printStackTrace(System.out);
            } else if (expected == Boolean.TRUE) {
                assertDoesNotThrow(parser::parse);
            } else {
                try {
                    parser.parse();
                    System.out.println("accepted");
                } catch (JsonParserException ex) {
                    System.out.println("rejected");
                    ex.printStackTrace(System.out);
                }
            }
        }
    }

    static @NotNull Stream<@NotNull Arguments> parameters() throws IOException {
        var filenames = new ArrayList<Arguments>();

        try (
                InputStream in = Objects.requireNonNull(JsonParserTest.class.getResource("/nst/JsonTestSuite")).openStream();
                BufferedReader br = new BufferedReader(new InputStreamReader(in))
        ) {
            String resource;
            while ((resource = br.readLine()) != null) {
                filenames.add(Arguments.of(resource));
            }
        }

        return filenames.stream();
    }
}
