package eu.jonahbauer.json.query;

import eu.jonahbauer.json.JsonValue;
import eu.jonahbauer.json.query.impl.Generator;
import eu.jonahbauer.json.query.parser.JQParser;
import eu.jonahbauer.json.query.parser.ast.JQProgram;
import lombok.AccessLevel;
import lombok.RequiredArgsConstructor;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.io.IOException;
import java.io.UncheckedIOException;

@RequiredArgsConstructor(access = AccessLevel.PRIVATE)
public class JsonQuery {
    private final @NotNull JQProgram programm;

    public static @NotNull JsonQuery parse(@NotNull String query) {
        try {
            var parser = new JQParser(query);
            var programm = parser.parseTopLevel();
            if (programm.expression() == null) throw new IllegalArgumentException();
            return new JsonQuery(programm);
        } catch (IOException ex) {
            throw new UncheckedIOException(ex);
        }
    }

    public @NotNull Generator<@NotNull JsonValue> run(@Nullable JsonValue value) {
        return programm.run(value);
    }
}
