package eu.jonahbauer.json.query;

import java.util.LinkedHashMap;
import java.util.SequencedMap;

import static eu.jonahbauer.json.JsonTemplateProcessor.JSON;

public class Main {
    public static void main(String[] args) {
        System.out.println();
        System.out.println("Object Identifier-Index: .foo, .foo.bar");

        JsonQuery.parse(".foo")
                .run(JSON."{\"foo\": 42, \"bar\": \"less interesting data\"}")
                .forEach(System.out::println);

        JsonQuery.parse(".foo")
                .run(JSON."{\"notfoo\": true, \"alsonotfoo\": false}")
                .forEach(System.out::println);

        JsonQuery.parse(".[\"foo\"]")
                .run(JSON."{\"foo\": 42}")
                .forEach(System.out::println);

        JsonQuery.parse(".foo?")
                .run(JSON."{\"foo\": 42, \"bar\": \"less interesting data\"}")
                .forEach(System.out::println);

        System.out.println();
        System.out.println("Optional Object Identifier-Index: .foo?");

        JsonQuery.parse(".foo?")
                .run(JSON."{\"notfoo\": true, \"alsonotfoo\": false}")
                .forEach(System.out::println);

        JsonQuery.parse(".[\"foo\"]?")
                .run(JSON."{\"foo\": 42}")
                .forEach(System.out::println);

        JsonQuery.parse("[.foo?]")
                .run(JSON."[1, 2]")
                .forEach(System.out::println);

        System.out.println();
        System.out.println("Array Index: .[<number>]");

        JsonQuery.parse(".[0]")
                .run(JSON."[{\"name\":\"JSON\", \"good\":true}, {\"name\":\"XML\", \"good\":false}]")
                .forEach(System.out::println);

        JsonQuery.parse(".[2]")
                .run(JSON."[{\"name\":\"JSON\", \"good\":true}, {\"name\":\"XML\", \"good\":false}]")
                .forEach(System.out::println);

        JsonQuery.parse(".[-2]")
                .run(JSON."[1, 2, 3]")
                .forEach(System.out::println);

        System.out.println();
        System.out.println("Array/String Slice: .[<number>:<number>]");

        JsonQuery.parse(".[2:4]")
                .run(JSON."[\"a\",\"b\",\"c\",\"d\",\"e\"]")
                .forEach(System.out::println);

        JsonQuery.parse(".[2:4]")
                .run(JSON."\"abcdefghi\"")
                .forEach(System.out::println);

        JsonQuery.parse(".[:3]")
                .run(JSON."[\"a\",\"b\",\"c\",\"d\",\"e\"]")
                .forEach(System.out::println);

        JsonQuery.parse(".[-2:]")
                .run(JSON."[\"a\",\"b\",\"c\",\"d\",\"e\"]")
                .forEach(System.out::println);

        System.out.println();
        System.out.println("Array/Object Value Iterator: .[]");

        JsonQuery.parse(".[]")
                .run(JSON."[{\"name\":\"JSON\", \"good\":true}, {\"name\":\"XML\", \"good\":false}]")
                .forEach(System.out::println);

        JsonQuery.parse(".[]")
                .run(JSON."[]")
                .forEach(System.out::println);

        JsonQuery.parse(".foo[]")
                .run(JSON."{\"foo\":[1,2,3]}")
                .forEach(System.out::println);

        JsonQuery.parse(".[]")
                .run(JSON."{\"a\": 1, \"b\": 1}")
                .forEach(System.out::println);

//        var root = JsonValue.valueOf(List.of(
//                List.of(
//                        List.of(10, 20),
//                        List.of(30, 40)
//                ),
//                List.of(
//                        List.of(1, 2),
//                        List.of(3, 4)
//                )
//        ));
//
//        var key = new JQCommaExpression(new JQConstant(new JsonNumber(0)), new JQConstant(new JsonNumber(1)));
//
//        var pattern = new ArrayPattern(List.of(
//                new JQAsExpression.Pattern.ObjectPattern(of(
//                        key, new JQAsExpression.Pattern.ObjectPattern(of(
//                                key,
//                                new JQAsExpression.Pattern.ValuePattern("$x")
//                        ))
//                )),
//                new JQAsExpression.Pattern.ObjectPattern(of(
//                        key, new JQAsExpression.Pattern.ObjectPattern(of(
//                                key,
//                                new JQAsExpression.Pattern.ValuePattern("$y")
//                        ))
//                ))
//        ));
//
//        var expression = new JQBinaryExpression(
//                new JQVariableExpression("$x"),
//                new JQVariableExpression("$y"),
//                JQBinaryExpression.Operator.ADD
//        );
//
//        var as = new JQAsExpression(new JQConstant(root), List.of(pattern), expression);
//        as.evaluate(new JQExpression.Context(null)).forEach(System.out::println);
//
//        var parser = new JQParser("[{(0, 1): {(0,1): $x}}, {(0, 1): {(0, 1): $y}}]");
//        var pattern2 = parser.parsePattern();
//        System.out.println(pattern);
//        System.out.println(pattern2);
    }

    private static <K, V> SequencedMap<K, V> of(K key1, V value1) {
        var map = new LinkedHashMap<K, V>();
        map.put(key1, value1);
        return map;
    }
}
