package eu.jonahbauer.json.query.impl;

import org.jetbrains.annotations.NotNull;

import java.util.LinkedList;
import java.util.List;
import java.util.NoSuchElementException;
import java.util.Queue;

final class ConcatGenerator<T> implements Generator<T> {
    private final @NotNull Queue<@NotNull Generator<? extends T>> generators;

    public ConcatGenerator(@NotNull List<@NotNull Generator<? extends T>> generators) {
        this.generators = new LinkedList<>(generators);
    }

    @Override
    public boolean hasNext() {
        while (!generators.isEmpty()) {
            var current = generators.peek();
            if (!current.hasNext()) {
                generators.remove();
            } else {
                return true;
            }
        }
        return false;
    }

    @Override
    public T next() {
        while (!generators.isEmpty()) {
            var current = generators.peek();
            if (!current.hasNext()) {
                generators.remove();
            } else {
                return current.next();
            }
        }
        throw new NoSuchElementException();
    }
}
