package eu.jonahbauer.json.query.impl;

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.NoSuchElementException;
import java.util.function.Predicate;

final class FilterGenerator<T> implements Generator<T> {
    private final @NotNull Generator<T> generator;
    private final @NotNull Predicate<? super T> filter;

    private @Nullable T value;
    private boolean hasValue;

    public FilterGenerator(@NotNull Generator<T> generator, @NotNull Predicate<? super T> filter) {
        this.generator = generator;
        this.filter = filter;
    }

    public boolean advance() {
        if (hasValue) return true;
        while (generator.hasNext()) {
            var value = generator.next();
            if (filter.test(value)) {
                this.value = value;
                this.hasValue = true;
                return true;
            }
        }
        this.value = null;
        this.hasValue = false;
        return false;
    }

    @Override
    public boolean hasNext() {
        return advance();
    }

    @Override
    public T next() {
        if (!advance()) throw new NoSuchElementException();
        var out = value;
        hasValue = false;
        value = null;
        return out;
    }
}
