package eu.jonahbauer.json.query.impl;

import org.jetbrains.annotations.NotNull;

import java.util.function.Function;

final class FlatMappingGenerator<T, S> implements Generator<S> {
    private final @NotNull Generator<T> source;
    private final @NotNull Function<? super T, ? extends Generator<? extends S>> function;
    private @NotNull Generator<? extends S> current = Generator.empty();

    public FlatMappingGenerator(@NotNull Generator<T> source, @NotNull Function<? super T, ? extends Generator<? extends S>> function) {
        this.source = source;
        this.function = function;
    }

    @Override
    public boolean hasNext() {
        while (!current.hasNext() && source.hasNext()) {
            current = function.apply(source.next());
        }
        return current.hasNext();
    }

    @Override
    public S next() {
        while (!current.hasNext() && source.hasNext()) {
            current = function.apply(source.next());
        }
        return current.next();
    }
}
