package eu.jonahbauer.json.query.impl;

import org.jetbrains.annotations.NotNull;

import java.util.NoSuchElementException;
import java.util.Objects;

final class LimitGenerator<T> implements Generator<T> {
    private final @NotNull Generator<T> delegate;
    private int count;

    public LimitGenerator(@NotNull Generator<T> delegate, int count) {
        if (count < 0) throw new IllegalArgumentException();
        this.delegate = Objects.requireNonNull(delegate);
        this.count = count;
    }

    @Override
    public boolean hasNext() {
        return count != 0 && delegate.hasNext();
    }

    @Override
    public T next() {
        if (count == 0 || !delegate.hasNext()) throw new NoSuchElementException();
        count--;
        return delegate.next();
    }
}
