package eu.jonahbauer.json.query.impl;

import org.jetbrains.annotations.NotNull;

import java.util.LinkedList;
import java.util.NoSuchElementException;
import java.util.Objects;
import java.util.Queue;
import java.util.function.BiConsumer;
import java.util.function.Consumer;

final class MapMultiGenerator<T, S> implements Generator<S> {
    private final @NotNull Generator<T> generator;
    private final @NotNull BiConsumer<? super T, ? super Consumer<S>> mapping;
    private final @NotNull Queue<S> queue = new LinkedList<>();

    public MapMultiGenerator(@NotNull Generator<T> generator, @NotNull BiConsumer<? super T, ? super Consumer<S>> mapping) {
        this.generator = Objects.requireNonNull(generator);
        this.mapping = Objects.requireNonNull(mapping);
    }

    private boolean advance() {
        while (queue.isEmpty() && generator.hasNext()) {
            mapping.accept(generator.next(), (Consumer<S>) queue::add);
        }
        return !queue.isEmpty();
    }

    @Override
    public boolean hasNext() {
        return advance();
    }

    @Override
    public S next() {
        if (!advance()) throw new NoSuchElementException();
        return queue.remove();
    }
}
