package eu.jonahbauer.json.query.parser.ast;

import eu.jonahbauer.json.JsonValue;
import eu.jonahbauer.json.query.JsonMath;
import eu.jonahbauer.json.query.impl.Generator;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.NoSuchElementException;
import java.util.Objects;

@SuppressWarnings("preview")
public record JQAlternativeExpression(@NotNull JQExpression first, @NotNull JQExpression second) implements JQExpression {
    public JQAlternativeExpression {
        Objects.requireNonNull(first);
        Objects.requireNonNull(second);
    }

    @Override
    public @NotNull Generator<@Nullable JsonValue> evaluate(@NotNull Context context) {
        var first = this.first.evaluate(context);
        var second = this.second.evaluate(context);
        return new Generator<>() {
            private boolean empty = true;

            private JsonValue value;
            private boolean hasValue;

            private boolean advance() {
                if (hasValue) return true;
                while (first.hasNext()) {
                    var value = first.next();
                    if (JsonMath.isTruthy(value)) {
                        this.empty = false;
                        this.value = value;
                        this.hasValue = true;
                        return true;
                    }
                }
                if (empty && second.hasNext()) {
                    this.value = second.next();
                    this.hasValue = true;
                    return true;
                }
                return false;
            }

            @Override
            public boolean hasNext() {
                return advance();
            }

            @Override
            public @Nullable JsonValue next() throws NoSuchElementException {
                if (!advance()) throw new NoSuchElementException();
                var out = value;
                hasValue = false;
                value = null;
                return out;
            }
        };
    }

    @Override
    public boolean isConstant() {
        return first.isConstant() && second.isConstant();
    }

    @Override
    public @NotNull String toString() {
        return first + " // " + second;
    }
}
