package eu.jonahbauer.json.query.parser.ast;

import eu.jonahbauer.json.JsonValue;
import eu.jonahbauer.json.query.impl.Generator;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.Objects;

public record JQAsExpression(
        @NotNull JQExpression variable, @NotNull JQPatterns patterns, @NotNull JQExpression expression
) implements JQExpression {

    public JQAsExpression {
        Objects.requireNonNull(variable);
        Objects.requireNonNull(expression);
        Objects.requireNonNull(patterns);
    }

    @Override
    public @NotNull Generator<@Nullable JsonValue> evaluate(@NotNull Context context) {
        return variable.evaluate(context)
                .flatMap(value -> patterns.bind(context, value).map(context::withVariables).flatMap(expression::evaluate));
    }

    @Override
    public boolean isConstant() {
        return variable.isConstant() && expression.isConstant();
    }

    @Override
    public @NotNull String toString() {
        return variable + " as " + patterns + " | " + expression;
    }

}
