package eu.jonahbauer.json.query.parser.ast;

import eu.jonahbauer.json.JsonValue;
import eu.jonahbauer.json.query.JsonMath;
import eu.jonahbauer.json.query.impl.Generator;
import lombok.RequiredArgsConstructor;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.Objects;
import java.util.function.BinaryOperator;
import java.util.stream.Stream;

public record JQAssignment(@NotNull JQExpression target, @NotNull JQExpression value, @NotNull Operator operator) implements JQExpression {
    public static @NotNull JQAssignment add(@NotNull JQExpression target, @NotNull JQExpression value) {
        return new JQAssignment(target, value, Operator.ADD);
    }

    public static @NotNull JQAssignment sub(@NotNull JQExpression target, @NotNull JQExpression value) {
        return new JQAssignment(target, value, Operator.SUB);
    }

    public static @NotNull JQAssignment mul(@NotNull JQExpression target, @NotNull JQExpression value) {
        return new JQAssignment(target, value, Operator.MUL);
    }

    public static @NotNull JQAssignment div(@NotNull JQExpression target, @NotNull JQExpression value) {
        return new JQAssignment(target, value, Operator.DIV);
    }

    public static @NotNull JQAssignment mod(@NotNull JQExpression target, @NotNull JQExpression value) {
        return new JQAssignment(target, value, Operator.MOD);
    }

    public JQAssignment {
        Objects.requireNonNull(target);
        Objects.requireNonNull(value);
        Objects.requireNonNull(operator);
    }

    public JQAssignment(@NotNull JQExpression target, @NotNull JQExpression value) {
        this(target, value, Operator.ID);
    }

    @Override
    public @NotNull Generator<JsonValue> evaluate(@NotNull Context context) {
        throw new UnsupportedOperationException("not yet implemented");
    }

    @Override
    public boolean isConstant() {
        throw new UnsupportedOperationException("not yet implemented");
    }

    @Override
    public @NotNull String toString() {
        return STR."\{target} \{operator.symbol} \{value}";
    }

    @RequiredArgsConstructor
    public enum Operator {
        ID((_, value) -> value, "="),
        ADD(JsonMath::add, "+="),
        SUB(JsonMath::sub, "-="),
        MUL(JsonMath::mul, "*="),
        DIV(JsonMath::div, "/="),
        MOD(JsonMath::mod, "%="),
        ;

        private final @NotNull BinaryOperator<@Nullable JsonValue> operator;
        private final @NotNull String symbol;
    }
}
