package eu.jonahbauer.json.query.parser.ast;

import eu.jonahbauer.json.JsonValue;
import eu.jonahbauer.json.query.JsonMath;
import eu.jonahbauer.json.query.impl.Generator;
import eu.jonahbauer.json.query.util.Util;
import lombok.RequiredArgsConstructor;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.List;
import java.util.Objects;
import java.util.function.BinaryOperator;

public record JQBinaryExpression(@NotNull JQExpression first, @NotNull JQExpression second, @NotNull Operator operator) implements JQExpression {
    public static @NotNull JQBinaryExpression add(@NotNull JQExpression first, @NotNull JQExpression second) {
        return new JQBinaryExpression(first, second, Operator.ADD);
    }

    public static @NotNull JQBinaryExpression sub(@NotNull JQExpression first, @NotNull JQExpression second) {
        return new JQBinaryExpression(first, second, Operator.SUB);
    }

    public static @NotNull JQBinaryExpression mul(@NotNull JQExpression first, @NotNull JQExpression second) {
        return new JQBinaryExpression(first, second, Operator.MUL);
    }

    public static @NotNull JQBinaryExpression div(@NotNull JQExpression first, @NotNull JQExpression second) {
        return new JQBinaryExpression(first, second, Operator.DIV);
    }

    public static @NotNull JQBinaryExpression mod(@NotNull JQExpression first, @NotNull JQExpression second) {
        return new JQBinaryExpression(first, second, Operator.MOD);
    }

    public static @NotNull JQBinaryExpression eq(@NotNull JQExpression first, @NotNull JQExpression second) {
        return new JQBinaryExpression(first, second, Operator.EQ);
    }

    public static @NotNull JQBinaryExpression neq(@NotNull JQExpression first, @NotNull JQExpression second) {
        return new JQBinaryExpression(first, second, Operator.NEQ);
    }

    public static @NotNull JQBinaryExpression lt(@NotNull JQExpression first, @NotNull JQExpression second) {
        return new JQBinaryExpression(first, second, Operator.LT);
    }

    public static @NotNull JQBinaryExpression gt(@NotNull JQExpression first, @NotNull JQExpression second) {
        return new JQBinaryExpression(first, second, Operator.GT);
    }

    public static @NotNull JQBinaryExpression leq(@NotNull JQExpression first, @NotNull JQExpression second) {
        return new JQBinaryExpression(first, second, Operator.LEQ);
    }

    public static @NotNull JQBinaryExpression geq(@NotNull JQExpression first, @NotNull JQExpression second) {
        return new JQBinaryExpression(first, second, Operator.GEQ);
    }

    public JQBinaryExpression {
        Objects.requireNonNull(first);
        Objects.requireNonNull(second);
        Objects.requireNonNull(operator);
    }

    @Override
    public @NotNull Generator<@Nullable JsonValue> evaluate(@NotNull Context context) {
        return Util.crossReversed(List.of(first, second), context)
                .map(values -> operator.apply(values.getFirst(), values.getLast()));
    }

    @Override
    public boolean isConstant() {
        return first.isConstant() && second.isConstant();
    }

    @Override
    public @NotNull String toString() {
        return STR."\{first} \{operator.symbol} \{second}";
    }

    @RequiredArgsConstructor
    public enum Operator implements BinaryOperator<@Nullable JsonValue> {
        ADD(JsonMath::add, "+"),
        SUB(JsonMath::sub, "-"),
        MUL(JsonMath::mul, "*"),
        DIV(JsonMath::div, "/"),
        MOD(JsonMath::mod, "%"),
        EQ(JsonMath::eq, "=="),
        NEQ(JsonMath::neq, "!="),
        LT(JsonMath::lt, "<"),
        GT(JsonMath::gt, ">"),
        LEQ(JsonMath::leq, "<="),
        GEQ(JsonMath::geq, ">="),
        ;

        private final @NotNull BinaryOperator<@Nullable JsonValue> operator;
        private final @NotNull String symbol;

        @Override
        public @Nullable JsonValue apply(@Nullable JsonValue value, @Nullable JsonValue value2) {
            return operator.apply(value, value2);
        }
    }
}
