package eu.jonahbauer.json.query.parser.ast;

import eu.jonahbauer.json.JsonBoolean;
import eu.jonahbauer.json.JsonValue;
import eu.jonahbauer.json.query.JsonMath;
import eu.jonahbauer.json.query.impl.Generator;
import org.jetbrains.annotations.NotNull;

import java.util.Objects;

public record JQBooleanAndExpression(@NotNull JQExpression first, @NotNull JQExpression second) implements JQExpression {

    public JQBooleanAndExpression {
        Objects.requireNonNull(first);
        Objects.requireNonNull(second);
    }

    @Override
    public @NotNull Generator<JsonValue> evaluate(@NotNull Context context) {
        return first.evaluate(context)
                .flatMap(value -> JsonMath.isFalsy(value)
                        ? Generator.of(JsonBoolean.FALSE)
                        : second.evaluate(context).map(JsonMath::isTruthy).map(JsonBoolean::valueOf)
                );
    }

    @Override
    public boolean isConstant() {
        return first.isConstant() && second.isConstant();
    }

    @Override
    public @NotNull String toString() {
        return first + " and " + second;
    }
}
