package eu.jonahbauer.json.query.parser.ast;

import eu.jonahbauer.json.JsonValue;
import eu.jonahbauer.json.query.impl.Generator;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.NoSuchElementException;

public record JQForEachExpression(
        @NotNull JQExpression expression,
        @NotNull JQPatterns patterns,
        @NotNull JQExpression init,
        @NotNull JQExpression update,
        @NotNull JQExpression extract
) implements JQExpression {

    @Override
    public @NotNull Generator<@Nullable JsonValue> evaluate(@NotNull Context context) {
        return init.evaluate(context)
                .flatMap(s -> new Generator<>() {
                    private final Generator<JsonValue> source = expression.evaluate(context);
                    private @NotNull Generator<JsonValue> current = Generator.empty();
                    private @Nullable JsonValue state = s;

                    private @NotNull Generator<JsonValue> advance() {
                        var item = source.next();

                        return patterns.bind(context, item)
                                .map(context::withVariables)
                                .map(ctx -> ctx.withRoot(state))
                                .flatMap(ctx -> update.evaluate(ctx).map(s -> {
                                    state = s;
                                    return ctx.withRoot(s);
                                }).flatMap(extract::evaluate));
                    }

                    @Override
                    public boolean hasNext() {
                        while (!current.hasNext() && source.hasNext()) current = advance();
                        return current.hasNext();
                    }

                    @Override
                    public @Nullable JsonValue next() throws NoSuchElementException {
                        while (!current.hasNext() && source.hasNext()) current = advance();
                        return current.next();
                    }
                });
    }

    @Override
    public boolean isConstant() {
        return expression.isConstant() && init.isConstant() && update.isConstant();
    }

    @Override
    public @NotNull String toString() {
        return "foreach " + expression + " as " + patterns + " (" + init + "; " + update + "; " + extract + ")";
    }
}
