package eu.jonahbauer.json.query.parser.ast;

import eu.jonahbauer.json.JsonValue;
import eu.jonahbauer.json.query.JsonQueryException;
import eu.jonahbauer.json.query.impl.Generator;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.*;

public record JQFunction(@NotNull String identifier, @NotNull List<@NotNull String> params, @NotNull JQExpression body) implements JQInvocable {
    public JQFunction {
        Objects.requireNonNull(identifier);
        Objects.requireNonNull(body);
        params = List.copyOf(params);
    }

    @Override
    public @NotNull Generator<@Nullable JsonValue> invoke(@NotNull JQExpression.Context context, @NotNull List<@NotNull JQExpression> arguments) {
        if (arguments.size() != params.size()) throw new JsonQueryException("invalid argument count");

        var expression = body;

        var functions = new ArrayList<JQFunction>();
        for (int i = params.size() - 1; i >= 0; i--) {
            String param = params.get(i);
            if (param.startsWith("$")) {
                expression = new JQAsExpression(
                        new JQFunctionInvocation(param.substring(1), List.of()),
                        new JQPatterns(List.of(new JQPatterns.Pattern.ValuePattern(param))),
                        expression
                );
                param = param.substring(1);
            }

            functions.add(new JQFunction(param, List.of(), arguments.get(i)));
        }

        return expression.evaluate(context.withFunctions(functions));

    }

    @Override
    public int arity() {
        return params().size();
    }

    @Override
    public @NotNull String toString() {
        return "def " + identifier + (params.isEmpty() ? "" : String.join("; ", params)) + ": " + body + ";";
    }
}
