package eu.jonahbauer.json.query.parser.ast;

import eu.jonahbauer.json.JsonValue;
import eu.jonahbauer.json.query.impl.Generator;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.List;
import java.util.Objects;
import java.util.StringJoiner;

public record JQFunctionInvocation(@NotNull String name, @NotNull List<@NotNull JQExpression> args) implements JQExpression {
    public JQFunctionInvocation {
        Objects.requireNonNull(name);
        args = List.copyOf(args);
    }

    @Override
    public @NotNull Generator<@Nullable JsonValue> evaluate(@NotNull Context context) {
        var function = context.function(name, args.size());
        return function.invoke(context, args);
    }

    @Override
    public boolean isConstant() {
        return false;
    }

    @Override
    public @NotNull String toString() {
        var out = new StringJoiner("; ", name + "(", ")");
        out.setEmptyValue(name);
        for (var arg : args) out.add(arg.toString());
        return out.toString();
    }
}
