package eu.jonahbauer.json.query.parser.ast;

import eu.jonahbauer.json.JsonValue;
import eu.jonahbauer.json.query.JsonMath;
import eu.jonahbauer.json.query.impl.Generator;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.Objects;

public record JQIfExpression(@NotNull JQExpression condition, @NotNull JQExpression then, @Nullable JQExpression otherwise) implements JQExpression {
    public JQIfExpression {
        Objects.requireNonNull(condition);
        Objects.requireNonNull(then);
    }

    public JQIfExpression(@NotNull JQExpression condition, @NotNull JQExpression then) {
        this(condition, then, null);
    }

    @Override
    public @NotNull Generator<@Nullable JsonValue> evaluate(@NotNull Context context) {
        return condition.evaluate(context).flatMap(result -> JsonMath.isTruthy(result)
                ? then.evaluate(context)
                : (otherwise != null ? otherwise.evaluate(context) : Generator.empty())
        );
    }

    @Override
    public boolean isConstant() {
        return condition.isConstant() && then.isConstant() && (otherwise == null || otherwise.isConstant());
    }

    @Override
    public @NotNull String toString() {
        if (otherwise instanceof JQIfExpression) {
            return "if " + condition + " then " + then + " el" + otherwise;
        } else if (otherwise != null) {
            return "if " + condition + " then " + then + " else " + otherwise + " end";
        } else {
            return "if " + condition + " then " + then + " end";
        }
    }
}
