package eu.jonahbauer.json.query.parser.ast;

import eu.jonahbauer.json.JsonNumber;
import eu.jonahbauer.json.JsonString;
import eu.jonahbauer.json.JsonValue;
import eu.jonahbauer.json.query.JsonMath;
import eu.jonahbauer.json.query.JsonQueryException;
import eu.jonahbauer.json.query.impl.Generator;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.Objects;

public record JQIndexExpression(@NotNull JQExpression expression, @NotNull JQExpression index, boolean optional) implements JQExpression {
    public JQIndexExpression {
        Objects.requireNonNull(expression);
        Objects.requireNonNull(index);
    }

    public JQIndexExpression(@NotNull JQExpression expression, @Nullable JsonValue index, boolean optional) {
        this(expression, new JQConstant(index), optional);
    }

    public JQIndexExpression(@NotNull JQExpression expression, double index, boolean optional) {
        this(expression, new JsonNumber(index), optional);
    }

    public JQIndexExpression(@NotNull JQExpression expression, @NotNull String index, boolean optional) {
        this(expression, new JsonString(index), optional);
    }

    @Override
    public @NotNull Generator<@Nullable JsonValue> evaluate(@NotNull Context context) {
        return expression.evaluate(context).flatMap(value -> index.evaluate(context).mapMulti((index, downstream) -> {
            try {
                downstream.accept(JsonMath.index(value, index));
            } catch (JsonQueryException ex) {
                if (!optional) throw ex;
            }
        }));
    }

    @Override
    public boolean isConstant() {
        return expression.isConstant() && index.isConstant();
    }

    @Override
    public @NotNull String toString() {
        return expression + "[" + index + "]" + (optional ? "?" : "");
    }
}
