package eu.jonahbauer.json.query.parser.ast;

import eu.jonahbauer.json.JsonArray;
import eu.jonahbauer.json.JsonObject;
import eu.jonahbauer.json.JsonValue;
import eu.jonahbauer.json.query.impl.Generator;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

public record JQRecursionExpression() implements JQExpression {
    @Override
    public @NotNull Generator<@Nullable JsonValue> evaluate(@NotNull Context context) {
        return recurse(context.root());
    }

    private @NotNull Generator<@Nullable JsonValue> recurse(@Nullable JsonValue value) {
        return switch (value) {
            case JsonArray array -> Generator.concat(Generator.of(array), Generator.from(array).flatMap(this::recurse));
            case JsonObject object -> Generator.concat(Generator.of(object), Generator.from(object.values()).flatMap(this::recurse));
            case null, default -> Generator.of(value);
        };
    }

    @Override
    public boolean isConstant() {
        return false;
    }
}
