package eu.jonahbauer.json.query.parser.ast;

import eu.jonahbauer.json.JsonValue;
import eu.jonahbauer.json.query.impl.Generator;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

public record JQReduceExpression(
        @NotNull JQExpression expression,
        @NotNull JQPatterns patterns,
        @NotNull JQExpression init,
        @NotNull JQExpression update
) implements JQExpression {

    @Override
    public @NotNull Generator<@Nullable JsonValue> evaluate(@NotNull Context context) {
        return init.evaluate(context)
                .map(state -> {
                    var it = expression.evaluate(context);
                    while (it.hasNext()) {
                        var item = it.next();

                        var bindings = patterns.bind(context, item);
                        var binding = bindings.next();
                        while (bindings.hasNext()) {
                            binding = bindings.next();
                        }

                        JsonValue nextState = null;
                        var up = update.evaluate(context.withVariables(binding).withRoot(state));
                        while (up.hasNext()) {
                            nextState = up.next();
                        }
                        state = nextState;
                    }
                    return state;
                });
    }

    @Override
    public boolean isConstant() {
        return expression.isConstant() && init.isConstant() && update.isConstant();
    }

    @Override
    public @NotNull String toString() {
        return "reduce " + expression + " as " + patterns + " (" + init + "; " + update + ")";
    }
}
