package eu.jonahbauer.json.query.parser.ast;

import eu.jonahbauer.json.JsonString;
import eu.jonahbauer.json.JsonValue;
import eu.jonahbauer.json.query.impl.Generator;
import eu.jonahbauer.json.query.util.Util;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.List;
import java.util.stream.Stream;

@SuppressWarnings("preview")
public record JQStringInterpolation(
        @Nullable String format,
        @NotNull List<@NotNull String> fragments,
        @NotNull List<@NotNull JQExpression> values
) implements JQExpression {
    public JQStringInterpolation {
        fragments = List.copyOf(fragments);
        values = List.copyOf(values);
        if (fragments.size() != values.size() + 1) throw new IllegalArgumentException();
    }

    @Override
    public @NotNull Generator<@Nullable JsonValue> evaluate(@NotNull Context context) {
        return Util.crossReversed(values, context)
                .map(values -> StringTemplate.of(fragments, values.stream().map(JQStringInterpolation::toString).toList()))
                .map(STR::process)
                .map(JsonString::valueOf);
    }

    private static @NotNull String toString(@Nullable JsonValue value) {
        return value instanceof JsonString(var string) ? string : JsonValue.toJsonString(value);
    }

    @Override
    public boolean isConstant() {
        return format == null && values.stream().allMatch(JQExpression::isConstant);
    }

    @Override
    public @NotNull String toString() {
        var out = new StringBuilder();
        if (format != null) out.append("@").append(format);
        out.append("\"");

        var it1 = fragments.iterator();
        for (JQExpression value : values) {
            var fragment = JsonString.quote(it1.next());
            out.append(fragment, 1, fragment.length() - 1);
            out.append("\\(");
            out.append(value);
            out.append(")");
        }
        var last = JsonString.quote(it1.next());
        out.append(last, 1, last.length());

        return out.toString();
    }
}
