package eu.jonahbauer.json.query.util;

import eu.jonahbauer.json.*;
import eu.jonahbauer.json.query.impl.Generator;
import eu.jonahbauer.json.query.parser.ast.JQExpression;
import lombok.experimental.UtilityClass;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.function.Supplier;
import java.util.stream.Stream;

@UtilityClass
public class Util {

    public static @NotNull Generator<@NotNull List<JsonValue>> cross(@NotNull List<@NotNull JQExpression> expressions, @NotNull JQExpression.Context context) {
        return cross(asSupplier(expressions, context)).map(Collections::unmodifiableList);
    }

    public static <T> @NotNull Generator<@NotNull List<T>> cross(@NotNull List<@NotNull Supplier<@NotNull Generator<T>>> expressions) {
        if (expressions.isEmpty()) return Generator.of(new ArrayList<T>().reversed());

        return expressions.getFirst().get()
                .flatMap(value -> cross(expressions.subList(1, expressions.size()))
                            .map(list -> { list.addFirst(value); return list; })
                );
    }

    public static @NotNull Generator<@NotNull List<JsonValue>> crossReversed(@NotNull List<@NotNull JQExpression> expressions, @NotNull JQExpression.Context context) {
        return crossReversed(asSupplier(expressions, context)).map(Collections::unmodifiableList);
    }

    public static <T> @NotNull Generator<@NotNull List<T>> crossReversed(@NotNull List<@NotNull Supplier<@NotNull Generator<T>>> expressions) {
        if (expressions.isEmpty()) return Generator.of(new ArrayList<>());

        return expressions.getLast().get()
                .flatMap(value -> crossReversed(expressions.subList(0, expressions.size() - 1))
                        .map(list -> { list.addLast(value); return list; })
                );
    }

    private static @NotNull List<@NotNull Supplier<@NotNull Generator<JsonValue>>> asSupplier(@NotNull List<@NotNull JQExpression> expressions, @NotNull JQExpression.Context context) {
        var list = new ArrayList<Supplier<Generator<JsonValue>>>(expressions.size());
        expressions.forEach(expr -> list.add(() -> expr.evaluate(context)));
        return list;
    }

    public static @NotNull String type(@Nullable JsonValue value) {
        return switch (value) {
            case JsonArray _ -> "array";
            case JsonObject _ -> "object";
            case JsonNumber _ -> "number";
            case JsonString _ -> "string";
            case JsonBoolean _ -> "boolean";
            case null -> "null";
        };
    }

    public static @NotNull String value(@Nullable JsonValue value) {
        return JsonValue.toJsonString(value);
    }
}
